<?php

namespace Drupal\harmonize\Harmonizer\EntityHarmonizer\EntityFieldHarmonizer;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\harmonize\Harmonizer\Harmonizer;
use Drupal\harmonize\Service\Harmonize;

/**
 * EntityFieldHarmonizer for a given field.
 *
 * This object is used to return clean field data.
 *
 * @property \Drupal\harmonize\Service\Harmonize $harmonizeService
 *
 * @package Drupal\harmonize\Harmonizer\EntityHarmonizer\EntityFieldHarmonizer
 */
class EntityFieldHarmonizer extends Harmonizer implements EntityFieldHarmonizerInterface {

  /**
   * Field Data definition.
   *
   * Contains all of the field's data for the given entity.
   *
   * @var \Drupal\Core\Field\FieldItemListInterface
   */
  protected $fieldData;

  /**
   * Field Name.
   *
   * Contains the name of the field being harmonized.
   *
   * @var string
   */
  protected $fieldName;

  /**
   * The value(s) of the field.
   *
   * @var mixed
   */
  protected $fieldValues;

  /**
   * Contains the type of field in string format.
   *
   * @var string
   */
  protected $fieldType;

  /**
   * Contains the field definition declaration.
   *
   * @var \Drupal\Core\Field\FieldDefinitionInterface
   */
  protected $fieldDefinition;

  /**
   * Contains the Drupal storage of the field.
   *
   * @var \Drupal\Core\Field\FieldStorageDefinitionInterface
   */
  protected $fieldStorage;

  /**
   * Contains the field cardinality.
   *
   * @var int
   */
  protected $cardinality;

  /**
   * EntityFieldHarmonizer constructor.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $fieldData
   *   The field data of the field being harmonized.
   * @param \Drupal\harmonize\Service\Harmonize $harmonizeService
   *   Harmonize module's main service containing Drupal services.
   * @param string $type
   *   The type of harmonizer defined by a string key.
   * @param string $signature
   *   The signature of the harmonizer generated by the factory.
   */
  public function __construct(FieldItemListInterface $fieldData, Harmonize $harmonizeService, string $type, string $signature) {
    parent::__construct($fieldData, $harmonizeService, $type, $signature);

    $this->fieldData = $fieldData;
    $this->fieldName = $fieldData->getName();
    $this->fieldValues = $fieldData->getValue();
    $this->fieldDefinition = $fieldData->getFieldDefinition();
    $this->fieldType = $this->fieldDefinition->getType();
    $this->fieldStorage = $this->fieldDefinition->getFieldStorageDefinition();
    $this->cardinality = $this->fieldStorage->getCardinality();
  }

  /**
   * Get the field type for this harmonizer.
   *
   * @return string
   *   Return the field type of the field for this harmonizer.
   */
  public function getFieldType() : string {
    return $this->fieldType;
  }

  /**
   * Get the field data for this harmonizer.
   *
   * @return \Drupal\Core\Field\FieldItemListInterface
   *   Raw field data returned.
   */
  public function getFieldData() : FieldItemListInterface {
    return $this->fieldData;
  }

  /**
   * {@inheritdoc}
   */
  protected function getHarmonizedData() {

    // Check if this is a single-value field.
    if ($this->cardinality === 1) {
      return $this->process($this->fieldValues[0], 0);
    }

    // Array that will hold the field data.
    $data = [];

    // Process all field values.
    foreach ($this->fieldValues as $i => $value) {
      $deltaData = $this->process($value, $i);
      if ($deltaData !== NULL) {
        $data[] = $deltaData;
      }
    }

    return $data;
  }

  /**
   * {@inheritdoc}
   */
  public function isCacheable(): bool {
    return FALSE;
  }

  /**
   * Formats the data of the field in a clean format.
   *
   * @param array $value
   *   Array of values for the field attached to this harmonizer.
   * @param int $i
   *   The index of the field value.
   *
   * @return mixed
   *   Data of the field for this harmonizer in a clean format.
   */
  public function process(array $value, int $i) {
    // Send error for unhandled FieldHarmonizerType.
    if (!isset($value['value']) || !array_key_exists('value', $value)) {
      // Logs an error for an unhandled EntityFieldHarmonizer type.
      \Drupal::logger('harmonize')->warning(
        t(
          'Error: Unhandled EntityFieldHarmonizer type: <strong>@type</strong> (@i). A field is not being harmonized correctly. Check with backend team to fix this by adding a new EntityFieldHarmonizer class that will handle this field type.',
          [
            '@type' => $this->fieldType,
            '@i' => $i,
          ]
        )
      );
      return NULL;
    }

    return $value['value'];
  }

}
