<?php

namespace Drupal\harmonize\Harmonizer\EntityHarmonizer;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\field\FieldConfigInterface;
use Drupal\harmonize\Constants\HarmonizeConfig;
use Drupal\harmonize\Constants\HarmonizerFlags;
use Drupal\harmonize\Constants\HarmonizerTypes;
use Drupal\harmonize\Harmonizer\EntityHarmonizer\EntityFieldHarmonizer\EntityFieldHarmonizerFactory;
use Drupal\harmonize\Harmonizer\Harmonizer;
use Drupal\harmonize\Service\Harmonize;

/**
 * Harmonizer for a given entity.
 *
 * This object is used to return clean field data from an Entity.
 *
 * @property Harmonize $harmonizeService
 *
 * @package Drupal\harmonize\Harmonizer\EntityHarmonizer
 */
class EntityHarmonizer extends Harmonizer implements EntityHarmonizerInterface {

  /**
   * Entity attached to the Harmonizer.
   *
   * @var \Drupal\Core\Entity\EntityInterface
   */
  protected $entity;

  /**
   * Contains the Entity Type Id.
   *
   * Examples: Node, Paragraph, Media.
   *
   * @var string
   */
  protected $entityTypeId;

  /**
   * Contains the Entity Bundle, if any.
   *
   * Examples: basic_page, basic_paragraph, etc.
   *
   * @var string
   */
  protected $entityBundle;

  /**
   * Array to store configurations for entities loaded from the site.
   *
   * @var array
   */
  protected $configurations;

  /**
   * Entity Harmonizer base constructor.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity being harmonized.
   * @param \Drupal\harmonize\Service\Harmonize $harmonizeService
   *   Harmonize module's main service containing Drupal services.
   * @param string $type
   *   The type of harmonizer defined by a string key.
   * @param string $signature
   *   The signature of the harmonizer generated by the factory.
   */
  public function __construct(
    EntityInterface $entity,
    Harmonize $harmonizeService,
    string $type,
    string $signature) {

    parent::__construct($entity, $harmonizeService, $type, $signature);

    $this->entity = $entity;
    $this->entityTypeId = $entity->getEntityType()->id();
    $this->entityBundle = $entity->bundle();
    $this->configurations = $this->getRelevantConfigurations();
  }

  /**
   * Get Entity Type ID for this Harmonizer.
   *
   * @return string
   *   Return Entity Type ID string.
   */
  public function getEntityTypeId() : string {
    return $this->entityTypeId;
  }

  /**
   * Get Entity Bundle for this Harmonizer.
   *
   * @return string
   *   Return Entity Bundle string.
   */
  public function getEntityBundle() : string {
    return $this->entityBundle;
  }

  /**
   * Get Entity for this Harmonizer.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   Return Entity.
   */
  public function getEntity() : EntityInterface {
    return $this->entity;
  }

  /**
   * {@inheritdoc}
   */
  protected function getHarmonizedData() {
    // Data array that will be returned.
    $data = [];

    // For now, we only go through with the rest if we're in a ContentEntity.
    // We will properly segment this later on for different types of entities.
    if (!($this->entity instanceof ContentEntityInterface)) {
      return $data;
    }

    // First, we want to set the harmonizer to the appropriate language.
    // Translate the entity to the appropriate language if a flag is set.
    // Otherwise, translate it to the current language.
    // We only do this if the entity is a content entity.
    $this->entity = isset($this->flags[HarmonizerFlags::LANG]) && !empty($this->flags[HarmonizerFlags::LANG]) ? $this->harmonizeService->helpers->ensureTranslated($this->entity, $this->flags[HarmonizerFlags::LANG]) : $this->harmonizeService->helpers->ensureTranslated($this->entity);


    // Add general information about the entity.
    $data['id'] = $this->getEntity()->id();
    $data['bundle'] = $this->getEntity()->bundle();

    $field_configs = $this->harmonizeService->helpers->getEntityTypeBundleFieldDefinitions($this->getEntityTypeId(), $this->getEntityBundle());

    // If this entity is being harmonized in a specified style...
    if (isset($this->flags[HarmonizerFlags::STYLE])) {
      // We filter our fields that must be filtered.
      $this->filterStyleFields($field_configs);
    }

    // Process all fields for the given entity.
    /* @var integer $key */
    /* @var \Drupal\field\FieldConfigInterface $field_config */
    foreach ($field_configs as $key => $field_config) {
      // If there are any custom filtering settings for this entity...
      $field_name = $field_config->getName();

      // If this field is configured to be excluded, skip it in this loop.
      if ($this->fieldIsExcluded($field_name)) {
        continue;
      }

      // Remove "field_" prefix if we have to.
      if ($this->harmonizeService->config[HarmonizeConfig::REMOVE_FIELD_UNDERSCORE_PREFIX] === 1) {
        $harmonized_field_key = \str_replace('field_', '', $field_name);
      }
      else {
        $harmonized_field_key = $field_name;
      }

      // Get the field data using the harmonizer.
      $harmonized_field_data = $this->getHarmonizedFieldData($field_config);

      // Set our field data to the array.
      $data[$harmonized_field_key] = $harmonized_field_data;
    }

    return $data;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags(): array {
    // Get the entity's normal cache tags and add tags specific to Harmonize.
    return array_merge($this->entity->getCacheTags(), [
      HarmonizerTypes::ENTITY,
      HarmonizerTypes::ENTITY . ':' . $this->entity->getEntityTypeId(),
      HarmonizerTypes::ENTITY . ':' . $this->entity->getEntityTypeId() . ':' . $this->entityBundle,
      HarmonizerTypes::ENTITY . ':' . $this->entity->getEntityTypeId() . ':' . $this->entityBundle . ':' . $this->entity->id(),
    ]);
  }

  /**
   * Harmonize the given Field and return the appropriate data.
   *
   * @param \Drupal\field\FieldConfigInterface $field_config
   *   Field Definition of the field being harmonized.
   *
   * @return mixed
   *   Return the field data in a proper, uniform format.
   */
  public function getHarmonizedFieldData(FieldConfigInterface $field_config) {
    // Get the name of the field.
    $field_name = $field_config->getName();

    // Get the field data of the entity's field.
    /* @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $this->getEntity();
    $entity_field_data = $entity->get($field_name);

    // If the data is empty, return null.
    // @todo - You can manage this in each EntityFieldHarmonizer by checking if the field is empty there instead.
    // @todo - This will give more flexibility as it will allow to send different empty values depending on the field.
    if ($entity_field_data->isEmpty()) {
      return [];
    }

    // Build field harmonizer.
    /* @var \Drupal\harmonize\Harmonizer\EntityHarmonizer\EntityFieldHarmonizer\EntityFieldHarmonizer $field_harmonizer */
    // If we have any flags here, the harmonizer must inherit some of them.
    $flags = [];
    if (!empty($this->flags)) {
      $flags = [
        HarmonizerFlags::LANG => $this->flags[HarmonizerFlags::LANG] ?? '',
      ];
    }
    $field_harmonizer = EntityFieldHarmonizerFactory::build($entity_field_data, $flags, $this->harmonizeService);

    // Harmonize the field data using the Harmonize module's main service.
    return $field_harmonizer->harmonize();
  }

  /**
   * Check if the given field is excluded for harmonization.
   *
   * @param string $field_name
   *   Name of the field to check.
   *
   * @return bool
   *   Returns true if the field is excluded. Returns false otherwise.
   */
  private function fieldIsExcluded(string $field_name): bool {
    // If configurations are empty, just exit.
    if (empty($this->configurations)) {
      return FALSE;
    }

    // Not excluded if configurations for this entity type do not exist.
    if (!isset($this->configurations[HarmonizeConfig::ENTITY_TYPE_EXCLUDED_FIELDS])) {
      return FALSE;
    }

    // Not excluded if configurations for this field do not exist.
    $entity_type_exclusion_config_exists = isset($this->configurations[HarmonizeConfig::ENTITY_TYPE_EXCLUDED_FIELDS][$field_name]);
    if ($entity_type_exclusion_config_exists) {
      $field_excluded_in_entity_type_exclusion_config = ($this->configurations[HarmonizeConfig::ENTITY_TYPE_EXCLUDED_FIELDS][$field_name] !== 0);
      if ($field_excluded_in_entity_type_exclusion_config) {
        return TRUE;
      }
    }

    // Check if bundle configurations exist and exclude this field.
    $entity_type_bundle_exclusion_config_exists = isset($this->configurations[$this->getEntityBundle()][HarmonizeConfig::ENTITY_TYPE_BUNDLE_EXCLUDED_FIELDS][$field_name]);
    if ($entity_type_bundle_exclusion_config_exists) {
      $field_excluded_in_entity_type_bundle_exclusion_config = ($this->configurations[$this->getEntityBundle()][HarmonizeConfig::ENTITY_TYPE_BUNDLE_EXCLUDED_FIELDS][$field_name] !== 0);
      if ($field_excluded_in_entity_type_bundle_exclusion_config) {
        return TRUE;
      }
    }

    // If all checks fail, return false.
    return FALSE;
  }

  /**
   * Filter fields that are to be harmonized for a given style.
   *
   * @param $field_config_array
   *   The field configuration array.
   */
  private function filterStyleFields(&$field_config_array) {
    $harmonyThemeConfig = $this->harmonizeService->getHarmonyThemeConfig();

    if (!$harmonyThemeConfig) {
      return;
    }

    if (!isset($harmonyThemeConfig['styles'])) {
      return;
    }

    $style = $this->getFlag(HarmonizerFlags::STYLE);
    if (!isset($harmonyThemeConfig['styles'][$style])) {
      return;
    }

    $styleFields = $harmonyThemeConfig['styles'][$style]['fields'];

    if (!empty($styleFields)) {
      $field_config_array = array_filter($field_config_array, function($fieldMachineName) use($styleFields) {
        return \in_array($fieldMachineName, $styleFields, TRUE);
      }, ARRAY_FILTER_USE_KEY);
    }
  }

  /**
   * Get custom harmonizer configurations for this harmonizer.
   *
   * This will return which fields are configured to be harmonized.
   *
   * @return array
   *   Array containing configuration values for this entity.
   */
  private function getRelevantConfigurations() : ?array {
    $config = \Drupal::config(HarmonizeConfig::DATA_CONFIG_NAME)->get(
      HarmonizeConfig::ENTITY_TYPE_SETTINGS
    );

    return $config[$this->getEntityTypeId()] ?? [];
  }

}
