<?php

namespace Drupal\harmonize\Harmonizer\FormHarmonizer;

use Drupal\harmonize\Harmonizer\Harmonizer;
use Drupal\harmonize\Constants\HarmonizerTypes;
use Drupal\harmonize\Service\Harmonize;

/**
 * Provides a Harmonizer for Drupal Forms.
 *
 * @package Drupal\harmonize\Harmonizer\FormHarmonizer
 */
class FormHarmonizer extends Harmonizer {

  /**
   * Drupal Variables array of the form being harmonized.
   *
   * @var array
   */
  protected $form;

  /**
   * FormHarmonizer constructor.
   *
   * @param array $form
   *   The render array of the form object being harmonized.
   * @param \Drupal\harmonize\Service\Harmonize $harmonizeService
   *   Harmonize module's main service containing Drupal services.
   * @param string $type
   *   The type of harmonizer defined by a string key.
   * @param string $signature
   *   The signature of the harmonizer generated by the factory.
   */
  public function __construct(
    array $form,
    Harmonize $harmonizeService,
    string $type,
    string $signature) {
    parent::__construct($form, $harmonizeService, $type, $signature);

    $this->type = HarmonizerTypes::FORM;
    $this->form = $form;
  }

  /**
   * Gets the form from the current harmonizer.
   *
   * @return array
   *   Raw render array of the form associated to this harmonizer.
   */
  public function getForm() : array {
    return $this->form;
  }

  /**
   * {@inheritdoc}
   */
  protected function getHarmonizedData() {
    $action = $this->getForm()['#action'] ?? NULL;
    $method = $this->getForm()['#method'] ?? NULL;
    $elements = $this->getFormElements();

    $data = [
      'action'   => $action,
      'method'   => $method,
      'elements' => $elements,
    ];

    return $data;
  }

  /**
   * Parse variables array and get all form elements.
   *
   * @return array
   *   Returns all form elements properly separated.
   */
  public function getFormElements() : array {
    $elements = [];

    foreach ($this->getForm() as $id => $value) {
      if (!\is_array($value)) {
        continue;
      }

      if (isset($value['#input']) && $value['#input'] === TRUE) {
        $fieldType = $value['#type'];

        switch ($fieldType) {
          case 'input':
          case 'textarea':
          case 'textfield':
          case 'search':
          case 'password':
            $elements[$id] = $this->inputTextField($value);
            break;

          case 'radio':
          case 'select':
            $elements[$id] = $this->inputOptionsField($value);
            break;

          default:
            $elements[$id] = $value;
            break;

        }
      }
    }

    return $elements;
  }

  /**
   * Properly format (harmonize) form text field.
   *
   * @param array $field
   *   An array of the field attributes.
   *
   * @return array
   *   Formatted array of the attributes for the input field element.
   */
  private function inputTextField(array $field) : array {
    return [
      'type'       => 'input',
      'label'      => $field['#title'] ?? NULL,
      'attributes' => [
        'type' => $field['#type'] ?? NULL,
        'name' => $field['#name'] ?? NULL,
      ],
    ];
  }

  /**
   * Properly format (harmonize) input options field.
   *
   * @param array $field
   *   An array of the field attributes.
   *
   * @return array
   *   Formatted array of the attributes for the input field element.
   */
  private function inputOptionsField(array $field) : array {
    return [
      'type'    => $field['#type'],
      'options' => $this->formatOptions($field['#options']),
    ];
  }

  /**
   * Properly format (harmonize) form field format options.
   *
   * @param array $options
   *   Raw format options of a given field.
   *
   * @return array
   *   Return the format options of the form field.
   */
  public function formatOptions(array $options) : array {
    $formatted_options = [];
    foreach ($options as $key => $option) {
      $formatted_options[] = [
        'label'      => $option,
        'attributes' => [
          'value' => !\is_int($key) ? $key : strtolower($option),
        ],
      ];
    }
    return $formatted_options;
  }

}
