<?php

namespace Drupal\harmonize\Harmonizer;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\harmonize\Constants\HarmonizationEvents;
use Drupal\harmonize\Constants\HarmonizeConfig;
use Drupal\harmonize\Constants\HarmonizerFlags;
use Drupal\harmonize\Event\HarmonizationEvent;
use Drupal\harmonize\Event\HarmonizationEventFactory;
use Drupal\harmonize\Service\Harmonize;

/**
 * Provides a base Class for all Harmonizers.
 *
 * @package Drupal\harmonize\Harmonizer
 */
abstract class Harmonizer implements HarmonizerInterface {

  // Used to avoid problem where database serialization isn't possible.
  use DependencySerializationTrait;

  /**
   * Harmonizer signature, defined by the object being harmonized.
   *
   * @var string
   */
  protected $signature;

  /**
   * Set the type of harmonizer this is.
   *
   * @var string
   */
  protected $type;

  /**
   * Data obtained from the harmonizer after all pre-processing is complete.
   *
   * @var array
   */
  protected $data;

  /**
   * The object being harmonized.
   *
   * The type-hint is "mixed" since this could be anything out of Drupal's many
   * objects.
   *
   * @var mixed
   */
  protected $object;

  /**
   * Manager service for harmonizer functionality.
   *
   * @var \Drupal\harmonize\Service\Helpers
   */
  protected $harmonizeService;

  /**
   * Flags that can be set on a harmonizer to alter pre-processing behavior.
   *
   * For a collection of flags that can be set and their effects, check out the
   * HarmonizerFlags class.
   *
   * @var array
   *
   * @see \Drupal\harmonize\Constants\HarmonizerFlags
   */
  protected $flags;

  /**
   * Parent Harmonizer that called this harmonizer.
   *
   * If a harmonizer is called in the field of an existing one, then it is
   * considered the child of the existing harmonizer. This allows us to define
   * the depth at which at object is being harmonized.
   *
   * @var \Drupal\harmonize\Harmonizer\Harmonizer
   */
  protected $parentHarmonizer;

  /**
   * Harmonizer constructor.
   *
   * @param mixed $object
   *   The object being harmonized.
   * @param \Drupal\harmonize\Service\Harmonize $harmonizeService
   *   Harmonize module's main service containing Drupal services.
   * @param string $type
   *   The type of harmonizer defined by a string key.
   * @param string $signature
   *   The signature of the harmonizer generated by the factory.
   */
  public function __construct($object, Harmonize $harmonizeService, string $type, string $signature) {
    $this->object = $object;
    $this->harmonizeService = $harmonizeService;
    $this->type = $type;
    $this->signature = $signature;
    $this->flags = [];
  }

  /**
   * Pre-process all fields in this harmonizer and set it as the cached data.
   *
   * @param array $flags
   *   Optionally set flags on the fly to use in your Harmonizer.
   *
   * @return array|string|null
   *   The data of the harmonizer if any.
   */
  public function harmonize(array $flags = []) {
    // Check if cache was set for this harmonizer.
    if ($this->data !== NULL && !isset($this->flags[HarmonizerFlags::STYLE])) {
      return $this->data;
    }

    // Merge the flags provided with the flags already set.
    if ($flags) {
      $this->flags = array_merge($this->flags, $flags);
    }

    // Cache actions.
    $cid = NULL;
    if ($this->harmonizeService->config[HarmonizeConfig::CACHE_ENABLED] && $this->isCacheable() && $this->cacheEnabled()) {
      // Return the result if it's cached.
      $cid = $this->getSignature();
      $style = $this->getFlag(HarmonizerFlags::STYLE);
      if ($style !== NULL) {
        $cid = $cid . '.' . $style;
      }
      $cid = $cid . '.' . $this->harmonizeService->languageManager->getCurrentLanguage()->getId();
      $cachedData = $this->harmonizeService->cacheBackend->get($cid);
      if ($cachedData) {
        return $cachedData->data;
      }
    }

    // Get the harmonized data.
    $this->data = $this->getHarmonizedData();

    // Dispatch all Harmonization events.
    $this->dispatchHarmonizationEvents();

    // Cache the result using the harmonizer's signature.
    if ($cid && $this->harmonizeService->config[HarmonizeConfig::CACHE_ENABLED] && $this->isCacheable()  && $this->cacheEnabled()) {
      $this->harmonizeService->cacheBackend->set($cid, $this->data, CacheBackendInterface::CACHE_PERMANENT, $this->getCacheTags());
    }

    // Return the data.
    return $this->data;

  }

  /**
   * Get the initially harmonized data for the given object.
   *
   * This quite simply loops through the objects properties, fields or
   * other data accordingly, and outputs all of the data in clean array
   * format.
   *
   * @return mixed
   *   The data properly formatted from the object.
   */
  abstract protected function getHarmonizedData();

  /**
   * {@inheritdoc}
   */
  public function getCacheTags(): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function isCacheable(): bool {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function cacheEnabled(): bool {
    $signatureParts = $this->getSignatureParts(TRUE);
    foreach ($signatureParts as $part) {
      if (\in_array($part, $this->harmonizeService->config[HarmonizeConfig::CACHE_EXCLUSIONS], TRUE)) {
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getData() {
    return $this->data;
  }

  /**
   * {@inheritdoc}
   */
  public function setData($data) : void {
    $this->data = $data;
  }

  /**
   * {@inheritdoc}
   */
  public function getObject() {
    return $this->object;
  }

  /**
   * {@inheritdoc}
   */
  public function getType() : string {
    return $this->type;
  }

  /**
   * {@inheritdoc}
   */
  public function getFlag(string $key) : ?string {
    return $this->flags[$key] ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function hasFlag(string $key): bool {
    return \array_key_exists($key, $this->flags);
  }

  /**
   * {@inheritdoc}
   */
  public function setFlag(string $key, string $value): void {
    $this->flags[$key] = $value;
  }

  /**
   * {@inheritdoc}
   */
  public function setFlags(array $flags): void {
    $this->flags = $flags;
  }

  /**
   * {@inheritdoc}
   */
  public function getSignature() : string {
    return $this->signature;
  }

  /**
   * {@inheritdoc}
   */
  public function setSignature(string $signature) : void {
    // @todo - Regex to check the format of the signature to make sure it's good.
    $this->signature = $signature;
  }

  /**
   * {@inheritdoc}
   */
  public function getSignatureParts(bool $incremental_parts = FALSE) {

    // Explode into array of elements, using "." as a delimiter.
    // Example: "entity.node" becomes ['entity', 'node'].
    $parts = explode('.', $this->signature);

    // If the incremental parts flag isn't set, return the exploded array.
    if (!$incremental_parts) {
      return $parts;
    }

    // Otherwise, we'll create the desired incremental format.
    // @see this function's documentation for more information.
    $result = [];

    foreach ($parts as $i => $part) {
      $needle = $part;

      if ($i !== 0) {
        $needle = '';
        for ($c = 0; $c <= $i; $c++) {
          $needle .= $parts[$c];
          if ($c !== $i) {
            $needle .= '.';
          }
        }
      }
      $result[] = $needle;
    }

    // Return the formatted result.
    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function getParentHarmonizer() : ?Harmonizer {
    return $this->parentHarmonizer;
  }

  /**
   * {@inheritdoc}
   */
  public function setParentHarmonizer(Harmonizer $harmonizer) : void {
    $this->parentHarmonizer = $harmonizer;
  }

  /**
   * {@inheritdoc}
   */
  public function depth(): int {
    $depth = 0;
    $parent = $this->getParentHarmonizer();

    while ($parent !== NULL) {
      $depth++;
      $parent = $parent->getParentHarmonizer();
    }

    return $depth;
  }

  /**
   * Dispatch events related to this Harmonizer.
   *
   * This allows other modules to react on Harmonization events and alter data.
   */
  public function dispatchHarmonizationEvents() : void {
    // Dispatch the core event.
    $this->harmonizeService->eventDispatcher->dispatch(
      HarmonizationEvents::CORE, new HarmonizationEvent($this)
    );

    // Build info on the event to dispatch for this Harmonizer.
    $event_info = HarmonizationEventFactory::build($this);

    // Dispatch the Harmonizer Type specific event.
    $this->harmonizeService->eventDispatcher->dispatch($event_info['event_name'], $event_info['event_object']);
  }

}
