<?php

namespace Drupal\harmonize\Harmonizer\MenuHarmonizer;

use Drupal\harmonize\Harmonizer\Harmonizer;
use Drupal\Component\Utility\Html;
use Drupal\harmonize\Constants\HarmonizerTypes;
use Drupal\harmonize\Service\Harmonize;

/**
 * Provides a Harmonizer for Drupal menus.
 *
 * @package Drupal\harmonize\Harmonizer\MenuHarmonizer
 */
class MenuHarmonizer extends Harmonizer {

  /**
   * Contains the name of the Drupal menu being harmonized.
   *
   * @var string
   */
  public $menuName;

  /**
   * Contains the list of menu items that need to be harmonized.
   *
   * @var array
   */
  protected $items;

  /**
   * MenuHarmonizer constructor.
   *
   * @param array $variables
   *   The render array of the menu object being harmonized.
   * @param \Drupal\harmonize\Service\Harmonize $harmonizeService
   *   Harmonize module's main service containing Drupal services.
   * @param string $type
   *   The type of harmonizer defined by a string key.
   * @param string $signature
   *   The signature of the harmonizer generated by the factory.
   */
  public function __construct(
    array $variables,
    Harmonize $harmonizeService,
    string $type,
    string $signature) {
    parent::__construct($variables, $harmonizeService, $type, $signature);

    $this->type = HarmonizerTypes::MENU;
    $this->menuName = $variables['menu_name'] ?? $variables['#menu_name'];
    $this->items = $variables['items'] ?? $variables['#items'];
  }

  /**
   * Get the name of the Menu being harmonized.
   *
   * @return string
   *   Return the menu name of the current harmonizer.
   */
  public function getMenuName() : string {
    return $this->menuName;
  }

  /**
   * {@inheritdoc}
   */
  protected function getHarmonizedData() {
    // Array that will house all harmonized menu items.
    $links = [];

    foreach ($this->items as $id => $item) {
      $links[] = self::harmonizeMenuItem($item);
    }

    return $links;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags(): array {
    return [
      HarmonizerTypes::MENU,
      HarmonizerTypes::MENU . '.' . $this->menuName,
    ];
  }

  /**
   * Format a single menu item and output the data cleanly.
   *
   * We make this function public and static so that it can be used elsewhere.
   *
   * @param array $item
   *   Render array of the menu item being harmonized.
   *
   * @return array
   *   Data of the menu item in a clean format.
   */
  public static function harmonizeMenuItem(array $item) : array {
    $attributes = $item['url']->getOptions()['attributes'] ?? [];
    $attributes['href'] = $item['url']->toString();
    $plugin_id = $item['original_link']->getPluginId();
    $uuid = str_replace('menu_link_content:', '', $plugin_id);
    $entity_id = $item['original_link']->getRouteParameters()['node'] ?? '';
    $harmonizedItem = [
      'active' => $item['in_active_trail'] ?? FALSE,
      'label' => $item['title'],
      'description' => $item['original_link']->getDescription() ?? '',
      'menuId' => 'menu-' . Html::cleanCssIdentifier(strtolower($item['title'])),
      'inActiveTrail' => $item['in_active_trail'],
      'isExpanded' => $item['is_expanded'],
      'isCollapsed' => $item['is_collapsed'],
      'isExternal' => $item['url']->isExternal(),
      'pluginId' => $plugin_id,
      'uuid' => $uuid,
      'node' => $entity_id,
      'attributes' => $attributes,
      'subnav' => [],
    ];

    if ($item['url']->isExternal()) {
      $harmonizedItem['icon'] = 'dmt-icon-external';
    }

    if (isset($item['below']) && !empty($item['below'])) {
      foreach ($item['below'] as $subItem) {
        $harmonizedItem['subnav'][] = self::harmonizeMenuItem($subItem);
      }
    }

    return $harmonizedItem;
  }

}
