<?php

namespace Drupal\harmonize\Harmonizer\RegionHarmonizer;

use Drupal\harmonize\Harmonizer\Harmonizer;
use Drupal\harmonize\Constants\HarmonizerTypes;
use Drupal\harmonize\Service\Harmonize;

/**
 * Provides a Harmonizer for Drupal regions.
 *
 * @package Drupal\harmonize\Harmonizer\RegionHarmonizer
 */
class RegionHarmonizer extends Harmonizer {

  /**
   * Contains the name of the Drupal region being harmonized.
   *
   * @var string
   */
  public $regionName;

  /**
   * RegionHarmonizer constructor.
   *
   * @param array $variables
   *   The render array of the menu object being harmonized.
   * @param \Drupal\harmonize\Service\Harmonize $harmonizeService
   *   Harmonize module's main service containing Drupal services.
   * @param string $type
   *   The type of harmonizer defined by a string key.
   * @param string $signature
   *   The signature of the harmonizer generated by the factory.
   */
  public function __construct(
    array $variables,
    Harmonize $harmonizeService,
    string $type,
    string $signature) {
    parent::__construct($variables, $harmonizeService, $type, $signature);
    $this->type = HarmonizerTypes::REGION;
    $this->regionName = $variables['region'];
  }

  /**
   * Get the name of the Menu being harmonized.
   *
   * @return string
   *   Return the menu name of the current harmonizer.
   */
  public function getRegionName() : string {
    return $this->regionName;
  }

  /**
   * {@inheritdoc}
   */
  protected function getHarmonizedData() : array {

    // Initialize data array.
    $data = [];

    // Loop in the elements and harmonize them accordingly.
    // @TODO - Use the factory pattern here to harmonize types of Region entries.
    foreach ($this->object['elements'] as $key => $element) {

      // Handle Drupal's obnoxious element caching for regions.
      $element = $this->handleDrupalElementCache($key, $element);

      // If the element is an array and the plugin id is set...
      if (\is_array($element) && isset($element['#base_plugin_id'])) {
        // If this element is a menu block, we want to harmonize it.
        if ($element['#base_plugin_id'] === 'system_menu_block' || $element['#base_plugin_id'] === 'menu_block') {
          $data["menu_block_${key}"] = $this->harmonizeMenuBlockElement($element);
        }

        // If this element is a content or custom block, harmonize it.
        if ($element['#base_plugin_id'] === 'block_content') {
          $data["content_block_${key}"] = $this->harmonizeContentBlockElement($element);
        }

        // If this element is the system breadcrumbs block, harmonize it.
        if ($element['#base_plugin_id'] === 'system_breadcrumb_block') {
          $data["breadcrumb_block_${key}"] = $this->harmonizeBreadcrumbBlockElement($element);
        }
      }

      continue;
    }

    return $data;
  }

  /**
   * Harmonize the contents of a menu block placed inside of a Region.
   *
   * @param array $element
   *   Element variables expressed through an array.
   *
   * @return array
   *   Data of the menu block.
   */
  protected function harmonizeMenuBlockElement(array $element): array {
    return [
      // Can add more in here down the line.
      'title' => $element['#configuration']['label'] ?? '',
      'menu' => $this->harmonizeService->harmonize($element['content']),
    ];
  }

  /**
   * Harmonize the contents of a content block placed inside of a Region.
   *
   * @param array $element
   *   Element variables expressed through an array.
   *
   * @return array
   *   Data of the menu block.
   */
  protected function harmonizeContentBlockElement(array $element): array {
    // Get block entity.
    /* @var \Drupal\block_content\Entity\BlockContent $entity */
    $entity = $element['content']['#block_content'];

    // Initialize array of data we will return.
    $data = [];
    $data['data'] = $this->harmonizeService->harmonize($entity);
    $data['data']['title'] = $entity->label();
    return $data;
  }

  /**
   * Harmonize the contents of a system breadcrumb block inside of a Region.
   *
   * @param array $element
   *   Element variables expressed through an array.
   *
   * @return array
   *   Data of the menu block.
   */
  protected function harmonizeBreadcrumbBlockElement(array $element): array {

    // Initialize the array of links.
    $links = [];

    /* @var \Drupal\Core\Link $link */
    foreach ($element['content']['#links'] as $link) {
      $links[] = [
        'url' => $link->getUrl()->toString(),
        'text' => $link->getText(),
      ];
    }

    return $links;
  }

  /**
   * Note on Caching: Drupal has VERY aggressive caching for Harmonizers.
   *
   * Below, we apply a technique to cache the 'elements' of a Region in here
   * into our harmonizer cache bin. This allows us to always have a region's
   * elements available to play with. Other parts of a Region can be exempt
   * from this cache if they are not part of the region's 'elements'.
   *
   * Each Element has HEAVY FUCKING CACHING. We can't even disable em. So yeah.
   *
   * @param string $key
   *   The key of the region's element that will be obtained from the cache.
   * @param mixed $element
   *   Element to cache or get cache data for.
   *
   * @return mixed
   *   Return the element from the cache or the clean element.
   */
  protected function handleDrupalElementCache(string $key, $element) {
    // We need to cache the region's element.
    // Drupal has VERY aggressive caching on Regions that CAN'T be disabled.
    // See note above.
    $cid = $this->getSignature() . '_' . $key . '_' . 'element_cache' . '_' . $this->harmonizeService->languageManager->getCurrentLanguage()->getId();
    $cached = $this->harmonizeService->cacheBackend->get($cid);
    if ($cached) {
      return $cached->data;
    }

    $this->harmonizeService->cacheBackend->set($cid, $element);
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags(): array {
    return [
      HarmonizerTypes::REGION,
      HarmonizerTypes::REGION . '.' . $this->regionName,
    ];
  }

}
